@@ -6,10 +6,10 @@
 # Change 1..1 below to 1..last_test_to_print .
 # (It may become useful if the test is moved to ./t subdirectory.)
 
-use CGI ();
+use CGI::Switch ();
 use Apache::Constants;
 use strict qw(vars);
-my $q = CGI->new;
+my $q = CGI::Switch->new;
 $q->print($q->header(-type => "text/plain"));
 
 my $version = SERVER_VERSION; 
@@ -22,14 +22,14 @@ if($version =~ /1\.1\.\d/) {
 
 my(%SEEN, @export, $key, $val);
 while(($key,$val) = each %Apache::Constants::EXPORT_TAGS) {
-    #warn "importing tag $key\n";
+    warn "importing tag $key\n";
     Apache::Constants->import(":$key");
     push @export, grep {!$SEEN{$_}++} @$val;
 }
 
 push @export, grep {!$SEEN{$_}++} @Apache::Constants::EXPORT;
 
-my $tests = (1 + @export) - 4; 
+my $tests = (1 + @export) - 3; 
 print "1..$tests\n"; 
 #$loaded = 1;
 $q->print("ok 1\n");
@@ -37,11 +37,8 @@ my $ix = 2;
 
 my($sym);
 
-#skip some 1.3 stuff that 1.2 didn't have
-my %skip = map { $_,1 } qw(DONE REMOTE_DOUBLE_REV);
-
 for $sym (sort @export) {
-    next if $skip{$sym} or $sym =~ /SERVER_.*VERSION/;
+    next if $sym eq "DONE" or $sym =~ /SERVER_.*VERSION/;
     my $val = &$sym;
     $q->print(defined $val ? "" : "not ", "ok $ix ($sym: $val)\n");
     $ix++;
